@@ -263,9 +263,17 @@ class HklKeys:
         'reduce_behaviour': 'keep',
         'type': int
     }
+    __None = {
+        'default': '',
+        'description': 'Dummy column for irrelevant data',
+        'imperative': False,
+        'dtype': 'str',
+        'reduce_behaviour': 'keep',
+        'type': str
+    }
     defined_keys = {'h', 'k', 'l', 'F', 'I', 'si', 'sf', 'b', 'm', 'la', 'ph',
                     'u', 'r', 't', 'u1', 'u2', 'u3', 'v1', 'v2', 'v3',
-                    'x', 'y', 'z', 'ze', 'ze2', 'Iosi', 'Icsi', 'equiv'}
+                    'x', 'y', 'z', 'ze', 'ze2', 'Iosi', 'Icsi', 'equiv', 'None'}
     # TODO check if needed and cplt?
 
     def __init__(self, keys=()):
@@ -352,7 +360,7 @@ class HklFrame(BaseFrame):
     are defined and describe/operate on the :attr:`frame`.
     """
 
-    HKL_LIMIT = 99
+    HKL_LIMIT = 127
     """Highest absolute value of h, k or l index, which can be
     interpreted correctly by current version of the software."""
 
@@ -556,37 +564,41 @@ class HklFrame(BaseFrame):
             self.place()
         self._recalculate_structure_factors_and_intensities()
 
-    def fill(self, radius=2.0):
+    def fill(self, radius: float = 2.0) -> None:
         """
         Fill dataframe with all reflections within *radius* from space origin.
 
         :param radius: Maximum distance from the reciprocal space origin
             to placed reflection (in reciprocal Angstrom).
-        :type radius: float
         """
+        hkl_ratios = radius / np.array([self.a_r, self.b_r, self.c_r])
+        hkl_limits = np.ceil(hkl_ratios).astype(np.int16)
 
-        max_index = 25
+        def hkl_walls(h, k, l_):
+            hw = np.mgrid[h:h:1j, -k:k:2j*k+1j, -l_:l_:2j*l_+1j].reshape(3, -1)
+            kw = np.mgrid[-h:h:2j*h+1j, k:k:1j, -l_:l_:2j*l_+1j].reshape(3, -1)
+            lw = np.mgrid[-h:h:2j*h+1j, -k:k:2j*k+1j, l_:l_:1j].reshape(3, -1)
+            return hw.T, kw.T, lw.T
 
-        # make an initial guess of the hkl ball
-        def _make_hkl_ball(i=max_index):
-            hkl_grid = np.mgrid[-i:i:2j*i+1j, -i:i:2j*i+1j, -i:i:2j*i+1j]
-            hkls = np.stack(hkl_grid, -1).reshape(-1, 3)
-            xyz = np.array((self.a_w, self.b_w, self.c_w))
-            return hkls[lin.norm(hkls @ xyz, axis=1) <= radius]
-        hkl = _make_hkl_ball()
+        for _ in range(self.HKL_LIMIT):
+            limits_too_small = [any(lin.norm(hkls @ self.A_r, axis=1) <= radius)
+                                for hkls in hkl_walls(*hkl_limits)]
+            if any(limits_too_small):
+                hkl_limits += limits_too_small
+            else:
+                hkl_limits -= 1
+                break
 
-        # increase the ball size until all needed points are in
-        previous_length = -1
-        while len(hkl) > previous_length and max_index <= self.HKL_LIMIT:
-            previous_length = len(hkl)
-            max_index = max_index * 2
-            hkl = _make_hkl_ball(max_index)
+        if any(hkl_limits > self.HKL_LIMIT):
+            msg = 'Attempting to use hkl indices {} above HKL_LIMIT of {}'
+            raise ValueError(msg.format(hkl_limits, self.HKL_LIMIT))
 
-        # create new dataframe using obtained ball of data
-        _h, _k, _l = np.vsplit(hkl.T, 3)
+        hkls = np.indices(2 * hkl_limits + 1, np.int16).reshape(3, -1).T \
+            - hkl_limits
+        _h, _k, _l = hkls[lin.norm(hkls @ self.A_r, axis=1) <= radius].T
         ones = np.ones_like(np.array(_h)[0])
-        self.from_dict({'h': np.array(_h)[0], 'k': np.array(_k)[0],
-                        'l': np.array(_l)[0], 'I': ones, 'si': ones, 'm': ones})
+        self.from_dict({'h': np.squeeze(_h), 'k': np.squeeze(_k),
+                        'l': np.squeeze(_l), 'I': ones, 'si': ones, 'm': ones})
 
     def stats(self, bins=10, space_group=SG['P1']):
         """
@@ -956,6 +968,9 @@ class HklIo:
         | free_6   |          | h -4 k -4 l -4         | NO   | YES  | YES    |
         |          |          | I -8 si -8 m -4        |      | (a)  |        |
         +----------+----------+------------------------+------+------+--------+
+        | m80      |          | h 4 k 4 l 4 b 4 F 12   | YES  | YES  | NO     |
+        |          |          | None 132               | (b)  | (b)  |        |
+        +----------+----------+------------------------+------+------+--------+
         | shelx_2  | 2        | h 4 k 4 l 4 I 8 si 8   | NO   | YES  | NO     |
         |          |          | b 4 la 8               |      | (a)  |        |
         +----------+----------+------------------------+------+------+--------+
@@ -978,36 +993,36 @@ class HklIo:
         |          |          | Ic 14 I 14 si 13       | (*)  |      |        |
         +----------+----------+------------------------+------+------+--------+
         | tonto_F  |          | h -4 k -4 l -4         | YES  | YES  | YES    |
-        |          |          | F -8 sf -8             | (b)  | (b)  |        |
+        |          |          | F -8 sf -8             | (c)  | (c)  |        |
         +----------+----------+------------------------+------+------+--------+
         | tonto_I  | tonto    | h -4 k -4 l -4         | YES  | YES  | YES    |
-        |          |          | I -8 si -8             | (b)  | (b)  |        |
+        |          |          | I -8 si -8             | (c)  | (c)  |        |
         +----------+----------+------------------------+------+------+--------+
         | xd_F6    |          | h -4 k -4 l -4 b -3    | YES  | NO   | YES    |
-        |          |          | F -13 sf -13           | (c)  |      |        |
+        |          |          | F -13 sf -13           | (d)  |      |        |
         +----------+----------+------------------------+------+------+--------+
         | xd_F7    |          | h -4 k -4 l -4 b -3    | YES  | NO   | YES    |
-        |          |          | F -13 sf -13 t -10     | (c)  |      |        |
+        |          |          | F -13 sf -13 t -10     | (d)  |      |        |
         +----------+----------+------------------------+------+------+--------+
         | xd_F-7   |          | h -4 k -4 l -4 b -3    | YES  | NO   | YES    |
-        |          |          | F -13 sf -13 ph -10    | (c)  |      |        |
+        |          |          | F -13 sf -13 ph -10    | (d)  |      |        |
         +----------+----------+------------------------+------+------+--------+
         | xd_F13   |          | h -4 k -4 l -4 b -3    | YES  | NO   | YES    |
-        |          |          | F -13 sf -13 t -10     | (c)  |      |        |
+        |          |          | F -13 sf -13 t -10     | (d)  |      |        |
         |          |          | u1 -10 u2 -10 u3 -10   |      |      |        |
         |          |          | v1 -10 v2 -10 v3 -10   |      |      |        |
         +----------+----------+------------------------+------+------+--------+
         | xd_I6    | xd       | h -4 k -4 l -4 b -3    | YES  | NO   | YES    |
-        |          |          | I -13 si -13           | (c)  |      |        |
+        |          |          | I -13 si -13           | (d)  |      |        |
         +----------+----------+------------------------+------+------+--------+
         | xd_I7    |          | h -4 k -4 l -4 b -3    | YES  | NO   | YES    |
-        |          |          | I -13 si -13 t -10     | (c)  |      |        |
+        |          |          | I -13 si -13 t -10     | (d)  |      |        |
         +----------+----------+------------------------+------+------+--------+
         | xd_I-7   |          | h -4 k -4 l -4 b -3    | YES  | NO   | YES    |
-        |          |          | I -13 si -13 ph -10    | (c)  |      |        |
+        |          |          | I -13 si -13 ph -10    | (d)  |      |        |
         +----------+----------+------------------------+------+------+--------+
         | xd_I13   |          | h -4 k -4 l -4 b -3    | YES  | NO   | YES    |
-        |          |          | I -13 si -13 t -10     | (c)  |      |        |
+        |          |          | I -13 si -13 t -10     | (d)  |      |        |
         |          |          | u1 -10 u2 -10 u3 -10   |      |      |        |
         |          |          | v1 -10 v2 -10 v3 -10   |      |      |        |
         +----------+----------+------------------------+------+------+--------+
@@ -1022,11 +1037,13 @@ class HklIo:
 
         - Suffix (a) is a zero-line: a shelx ending line with h = k = l = 0,
 
-        - Prefix and suffix (b) are tonto-characteristic beginning/end of file,
+        - Prefix and suffix (b) express a superflip-style block/start end,
 
-        - Prefix (c) is an xd-characteristic line with info about file content.
+        - Prefix and suffix (c) are tonto-characteristic beginning/end of file,
 
-        Pre/suffixes denoted with (*) are not suported in terms of writing.
+        - Prefix (d) is an xd-characteristic line with info about file content.
+
+        Pre/suffixes denoted with (*) are not supported in terms of writing.
         A custom hkl file format can be defined by providing
         a *format string* instead of 'Name'.
         The string should look like the ones in column "contents".
@@ -1108,9 +1125,10 @@ class HklReader(HklIo):
         """
         slice_end = list(np.cumsum(self._format_dict['widths']))
         slice_beg = [0] + slice_end[:-1]
-        parsed = [line[beg:end] for beg, end in zip(slice_beg, slice_end)]
-        parsed = np.array(parsed)
         try:
+            parsed = [line[beg:end] for beg, end in zip(slice_beg, slice_end)]
+            parsed = np.array(parsed)
+            assert min([len(p) for p in parsed]) > 0
             assert len(parsed) == len(self._format_dict['widths'])
         except (ValueError, AssertionError):
             return None
