@@ -1,10 +1,15 @@
-from typing import Dict, Any
+from typing import Dict, Any, Iterable, Tuple, List
 
+import matplotlib.pyplot as plt
+from matplotlib import animation
+from mpl_toolkits.mplot3d import axes3d
 import numpy as np
+import numpy.linalg as lin
 from uncertainties import ufloat, ufloat_fromstr, unumpy, UFloat
 
 from hikari.dataframes import BaseFrame, CifFrame, UBaseFrame
-from hikari.utility import make_abspath, cfloat, det3x3, chemical_elements
+from hikari.utility import make_abspath, cfloat, det3x3, chemical_elements, \
+    rotation_around
 
 
 def calculate_similarity_indices(cif1_path: str,
@@ -193,7 +198,136 @@ def calculate_similarity_indices(cif1_path: str,
     terminate_output()
 
 
+def animate_similarity_index(u_diag: Iterable,
+                             transformations: List[np.ndarray],
+                             output_path: str) -> None:
+    """
+    Make a gif presenting the change of similarity index against some arbitrary
+    series of `transformations`. Initial displacement matrix must be diagonal,
+    values given in `u_diag`, evaluated against a regular unit cell with a = 1.
+
+    :param u_diag: triplet of u matrix diagonal elements
+    :param transformations: list of transformations to be plotted every 0.1 sec
+    :param output_path: path where resulting gif file will be saved
+    """
+    max_radius = 1.0
+
+    fig = plt.figure(figsize=(12, 12))  # Square figure
+
+    # Coefficients in a0 x**2 + a1 y**2 + a2 z**2 = 1 & corresponding radii
+    rx, ry, rz = np.sqrt(u_diag)
+    u = np.diag(u_diag)
+
+    # Set of all spherical angles:
+    v = np.linspace(0, 2 * np.pi, 100)
+    w = np.linspace(0, np.pi, 100)
+
+    # Cartesian coordinates that correspond to the spherical angles:
+    # (this is the equation of an ellipsoid):
+    x = rx * np.outer(np.cos(v), np.sin(w))
+    y = ry * np.outer(np.sin(v), np.sin(w))
+    z = rz * np.outer(np.ones_like(v), np.cos(w))
+
+    # set animation settings
+    fps = 10
+    steps = len(transformations)
+
+    def calculate_similarity_index(adp_frac_1, adp_frac_2) -> float or UFloat:
+        base_frame = BaseFrame()
+
+        def adp_frac2cart(adp_frac):
+            n = np.diag([base_frame.a_r, base_frame.b_r, base_frame.c_r])
+            return base_frame.A_d.T @ n @ adp_frac @ n @ base_frame.A_d
+        adp_cart_1 = adp_frac2cart(adp_frac_1)
+        adp_cart_2 = adp_frac2cart(adp_frac_2)
+        adp_inv_1 = np.linalg.inv(adp_cart_1)
+        adp_inv_2 = np.linalg.inv(adp_cart_2)
+        r12_num = 2 ** (3 / 2) * det3x3(adp_inv_1 @ adp_inv_2) ** (1 / 4)
+        r12_den = det3x3(adp_inv_1 + adp_inv_2) ** (1 / 2)
+        return 100 * (1 - r12_num / r12_den)
+
+    def get_xyz(step: int) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+        """Get positions of x, y, z nodes after `step` steps of animation"""
+        trans_matrix = transformations[step]
+        x_list = x.reshape(10_000)
+        y_list = y.reshape(10_000)
+        z_list = z.reshape(10_000)
+        xyz = np.vstack([x_list, y_list, z_list]).T
+        xyz_trans = xyz @ trans_matrix
+        x_trans = xyz_trans[:, 0].reshape(100, 100)
+        y_trans = xyz_trans[:, 1].reshape(100, 100)
+        z_trans = xyz_trans[:, 2].reshape(100, 100)
+        return x_trans, y_trans, z_trans
+
+    def get_u(step: int) -> np.ndarray:
+        """Get transformed u matrix after `step` steps of animation"""
+        trans_matrix = transformations[step]
+        return trans_matrix.T @ u @ trans_matrix
+
+    # prepare list of similarity indices
+    si = [calculate_similarity_index(u, get_u(i)) for i in range(steps)]
+    si.append(si[0])
+
+    def get_frame(step):
+        """Clear previous axes and produce new at `step` step of animation"""
+        for a in fig.axes:
+            a.clear()
+        x_, y_, z_ = get_xyz(step)
+        ax1 = fig.add_subplot(221, projection='3d')
+        ax2 = fig.add_subplot(222)
+        ax3 = fig.add_subplot(223, projection='3d')
+        ax4 = fig.add_subplot(224)
+        ax1.plot_surface(x, y, z, rstride=4, cstride=4, color='r')
+        ax2.plot(range(steps+1), si, 'b')
+        ax2.set_xlabel(step)
+        ax2.set_ylabel('similarity index')
+        ax2.axvline(x=step, color='b')
+        ax2.plot(step, si[step], 'bo')
+        ax2.set_xlim([0, steps])
+        ax2.set_ylim([0, 10])
+        ax3.plot_surface(x_, y_, z_, rstride=4, cstride=4, color='b')
+        u_ = get_u(step)
+        ax4_font = {'family': 'serif', 'weight': 'normal', 'size': 16}
+        ax4.clear()
+        ax4.axis('off')
+        ax4.text(0.2, 0.5, s='U = [', fontdict=ax4_font, ha='right')
+        ax4.text(0.4, 0.7, s=f'{u_[0, 0]:6.3f}', fontdict=ax4_font, ha='right')
+        ax4.text(0.6, 0.7, s=f'{u_[1, 0]:6.3f}', fontdict=ax4_font, ha='right')
+        ax4.text(0.8, 0.7, s=f'{u_[2, 0]:6.3f}', fontdict=ax4_font, ha='right')
+        ax4.text(0.4, 0.5, s=f'{u_[0, 1]:6.3f}', fontdict=ax4_font, ha='right')
+        ax4.text(0.6, 0.5, s=f'{u_[1, 1]:6.3f}', fontdict=ax4_font, ha='right')
+        ax4.text(0.8, 0.5, s=f'{u_[2, 1]:6.3f}', fontdict=ax4_font, ha='right')
+        ax4.text(0.4, 0.3, s=f'{u_[0, 2]:6.3f}', fontdict=ax4_font, ha='right')
+        ax4.text(0.6, 0.3, s=f'{u_[1, 2]:6.3f}', fontdict=ax4_font, ha='right')
+        ax4.text(0.8, 0.3, s=f'{u_[2, 2]:6.3f}', fontdict=ax4_font, ha='right')
+        ax4.text(0.9, 0.5, s=']', fontdict=ax4_font, ha='right')
+        for axis in 'xyz':
+            getattr(ax1, f'set_{axis}lim')((-max_radius, max_radius))
+            getattr(ax3, f'set_{axis}lim')((-max_radius, max_radius))
+
+    fig.tight_layout()
+    fig.subplots_adjust(left=0.02, bottom=0.02, right=0.98, top=0.98,
+                        wspace=None, hspace=None)
+    ani = animation.FuncAnimation(
+        fig=fig,
+        func=get_frame,
+        frames=steps,
+        interval=1000/fps,
+    )
+    ani_path = make_abspath(output_path)
+    ani.save(ani_path, writer='imagemagick', fps=fps)
+
+
 if __name__ == '__main__':
-    calculate_similarity_indices('~/_/si/1.cif',
-                                 '~/_/si/2.cif',
-                                 output_path='~/_/output.txt')
+    # calculate_similarity_indices('~/_/si/1.cif',
+    #                              '~/_/si/2.cif',
+    #                              output_path='~/_/si/output.txt')
+    t1 = [rotation_around(np.array([0, 0, 1]), by=np.deg2rad(10 * i))
+          for i in range(36)]
+    t2 = [np.eye(3) * np.sqrt((1.5 - np.cos(np.deg2rad(10 * i)) / 2))
+          for i in range(36)]
+    t3 = [np.array([(1, 0, 0), (0, 1, 0), (0, 0, np.sqrt(2 ** np.sin(np.deg2rad(10 * i))))])
+          for i in range(36)]
+    animate_similarity_index(u_diag=(2, 1, 1),
+                             transformations=t1,
+                             output_path='~/_/si/S_animation_211_zrot.gif')
