@@ -5,6 +5,8 @@ information / methods used in the package.
 
 import numpy as np
 import random
+import uncertainties
+from typing import Union
 
 
 def angle2rad(value):
@@ -34,13 +36,14 @@ def angle2rad(value):
     :return: Angle value expressed in radians
     :rtype: float
     """
-    return value if -3.15 < value < 3.15 else np.deg2rad(value)
+    return value if -3.15 < value < 3.15 else value * np.pi / 180
 
 
 def cart2sph(x, y, z):
     """
     Convert Cartesian coordinates x, y, z
     to conventional spherical coordinates r, p, a
+
     :param x: Cartesian coordinate or vector x
     :type x: float or np.ndarray
     :param y: Cartesian coordinate or vector y
@@ -60,6 +63,7 @@ def sph2cart(r, p, a):
     """
     Convert conventional spherical coordinates r, p, a
     to Cartesian coordinates x, y, z
+
     :param r: Spherical coordinate or vector radius
     :type r: float or np.ndarray
     :param p: Spherical coordinate or vector polar angle
@@ -75,6 +79,23 @@ def sph2cart(r, p, a):
     return np.array([x, y, z])
 
 
+def det3x3(matrix):
+    """
+    Calculate a determinant of a 3x3 matrix. Should be usually substituted
+    by `numpy.linalg.det`, but is indispensable for matrices with uncertainties.
+
+    :param matrix: 3x3 array/matrix which allows for 2d indexing
+    :type matrix: numpy.ndarray or uncertainties.unumpy.matrix
+    :return: Determinant of the matrix
+    :rtype: int or float or uncertainties.core.Variable
+    """
+    m11, m12, m13 = matrix[0, 0], matrix[1, 0], matrix[2, 0]
+    m21, m22, m23 = matrix[0, 1], matrix[1, 1], matrix[2, 1]
+    m31, m32, m33 = matrix[0, 2], matrix[1, 2], matrix[2, 2]
+    return m11 * m22 * m33 + m12 * m23 * m31 + m13 * m21 * m32 \
+        - m13 * m22 * m31 - m12 * m21 * m33 - m11 * m23 * m32
+
+
 def fibonacci_sphere(samples=1, seed=1337):
     """
     Return a 3D cartesian coordinates of *samples* number of points
@@ -127,12 +148,26 @@ def fibonacci_sphere(samples=1, seed=1337):
     return np.vstack([x, y, z]).T
 
 
-def euler_rodrigues_matrix(a, b, c, d):
+def euler_rodrigues_matrix(
+        a: Union[int, float, uncertainties.UFloat],
+        b: Union[int, float, uncertainties.UFloat],
+        c: Union[int, float, uncertainties.UFloat],
+        d: Union[int, float, uncertainties.UFloat]) -> np.ndarray:
+    """
+    Return a rotation matrix based on a Euler-Rodrigues parametrisation. For
+    details, see https://en.wikipedia.org/wiki/Euler%E2%80%93Rodrigues_formula.
+
+    :param a: Euler-Rodrigues parameter *a*
+    :param b: Euler-Rodrigues parameter *b*
+    :param c: Euler-Rodrigues parameter *c*
+    :param d: Euler-Rodrigues parameter *d*
+    :return: 3x3 matrix describing rotation
+    """
     aa, bb, cc, dd = a * a, b * b, c * c, d * d
     bc, ad, ac, ab, bd, cd = b * c, a * d, a * c, a * b, b * d, c * d
-    return np.array([[aa + bb - cc - dd, 2 * (bc + ad),     2 * (bd - ac)],
-                     [2 * (bc - ad),     aa + cc - bb - dd, 2 * (cd + ab)],
-                     [2 * (bd + ac),     2 * (cd - ab),     aa + dd - bb - cc]])
+    return np.array([[aa + bb - cc - dd, 2 * (bc + ad), 2 * (bd - ac)],
+                     [2 * (bc - ad), aa + cc - bb - dd, 2 * (cd + ab)],
+                     [2 * (bd + ac), 2 * (cd - ab), aa + dd - bb - cc]])
 
 
 def rotation_from(from_, to):
@@ -158,8 +193,9 @@ def rotation_from(from_, to):
             return np.eye(3)
         else:
             e1, e2 = np.array([1, 0, 0]), np.array([0, 1, 0])
-            e = e2 if np.isclose(sum(np.cross(e1, f)), 0.) else e1
-            return rotation_from(f, to=e) @ rotation_from(e, to=t)
+            e = np.cross(e2, f) if np.isclose(sum(np.cross(e1, f)), 0.) \
+                else np.cross(e1, f)
+            return rotation_around(e, by=np.pi)
     else:
         axis = axis / np.linalg.norm(axis)
         angle = np.arcsin(np.linalg.norm(np.cross(f, t)))
@@ -215,8 +251,8 @@ def weighted_quantile(values, quantiles, weights=None):
     values = np.array(values)
     quantiles = np.array(quantiles)
     w = np.ones_like(values) if weights is None else np.array(weights)
-    assert np.all(quantiles >= 0) and np.all(quantiles <= 1), \
-        'quantiles should be in [0, 1]'
+    if np.any(quantiles < 0) or np.any(quantiles > 1):
+        raise ValueError(f'quantiles ({quantiles}) should be in range 0 to 1')
 
     sorter = np.argsort(values)
     values = values[sorter]
